/*
** Module   : CommonProcedure.java
** Abstract : Defines common APIs used by the SESSION and PROCEDURE handles.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130112 Created initial version.
** 002 CS  20130218 Added SERVER, REMOTE, PROXY and TRANSACTION attributes.
** 003 CA  20130221 Moved to PersistentProcedure the procedure-only attributes/methods.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.util.ProcedureManager.SearchMode;

/**
 * Exposes the following methods or attributes, common for the SESSION and
 * PROCEDURE handles:
 * <table>
 * <tr>
 *    <th>attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>SUPER-PROCEDURES</td>
 *    <td>{@link #superProcedures()}</td>
 * </tr>
 * <tr>
 *    <td>ADD-SUPER-PROCEDURE</td>
 *    <td>addSuperProcedure APIs</td>
 * </tr>
 * <tr>
 *    <td>REMOVE-SUPER-PROCEDURE</td>
 *    <td>removeSuperProcedure APIs</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapCommonProcedure(handle)} API, which, depending on the 
 * handle's state and type, it either returns an 
 * {@link SessionUtils#asHandle()} instance (for SESSION handles) or an 
 * {@link ExternalProgramWrapper} instance (for PROCEDURE handles), or a 
 * dynamic proxy using {@link handle#invalidAttrAccessProxy}. 
 */
public interface CommonProcedure
{
   /**
    * Get the SUPER-PROCEDURES attribute for this SESSION or procedure handle.
    * 
    * @return   See above.
    */
   public character superProcedures();

   /**
    * Add the given handle as a super-procedure for the current handle.
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    */
   public logical addSuperProcedure(handle h);

   /**
    * Add the given handle as a super-procedure for the current handle, using
    * the explicit {@link SearchMode mode}.
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    * @param    mode
    *           The {@link SearchMode} to be set for the <code>h</code> 
    *           handle.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    */
   public logical addSuperProcedure(handle h, SearchMode mode);

   /**
    * Add the given handle as a super-procedure for the current handle, using
    * the search-mode set as a string (one of the <code>SEARCH-TARGET</code>
    * or <code>SEARCH-SELF</code> string literals).
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    * @param    mode
    *           The search-mode to be set for the <code>h</code> handle.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    */
   public logical addSuperProcedure(handle h, String mode);

   /**
    * Remove the given handle from this handle's list of super-procedures.
    *  
    * @param    h
    *           The procedure handle to be removed.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    */
   public logical removeSuperProcedure(handle h);
   
   /**
    * Indicates if the procedure is ran remote.
    * 
    * @return   <code>true</code> if the procedure is run remote , <code>false</code> otherwise.
    */
   public logical isRemote(); 
}